/**
 * JASMINe
 * Copyright (C) 2007-2012 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 */
package org.ow2.jasmine.monitoring.eventswitch.osgi;

import org.apache.felix.ipojo.annotations.Component;
import org.apache.felix.ipojo.annotations.Invalidate;
import org.apache.felix.ipojo.annotations.Requires;
import org.apache.felix.ipojo.annotations.Validate;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.mule.MuleManager;
import org.mule.config.ConfigurationBuilder;
import org.mule.config.ReaderResource;
import org.mule.config.builders.MuleXmlConfigurationBuilder;
import org.mule.umo.manager.UMOManager;
import org.ow2.jonas.jmx.JmxService;
import org.ow2.jonas.lib.bootstrap.JProp;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Wrapper class for Mule.
 */
@Component(name="MuleWrapper")
public class MuleWrapper {

    /**
     * The JOnAS JMX service.
     */
    @Requires
    private JmxService jmxService;

    /**
     * Logger.
     */
    protected static Log logger = LogFactory.getLog(MuleWrapper.class);

    private Thread muleStarterThread = null;

    /**
     * Configures Mule, which will automatically launch it.
     *
     * @throws Exception When something wrong happens ...
     */
    @Validate
    public void start() throws Exception {
        logger.debug("");

        // Get mbeanServer and j2eeServer
        MBeanServer mbeanServer = jmxService.getJmxServer();
        ObjectName j2eeServer = null;
        Set<?> j2eeServerNames = mbeanServer.queryNames(new ObjectName("*:j2eeType=J2EEServer,*"), null);
        if (j2eeServerNames.size() > 0) {
            j2eeServer = (ObjectName) j2eeServerNames.iterator().next();
        }
        if (j2eeServer == null) {
            throw new Exception("Failed locating the J2EEServer MBean");
        }

        // Wait on server running
        while (true) {
            String state = mbeanServer.getAttribute(j2eeServer, "state").toString();
            if (state.contains("running")) {
                break;
            }
            if (!state.contains("starting")) {
                throw new Exception("Server state is " + state + ", canceling JASMINe EventSwitch instance creation");
            }
            Thread.sleep(2000);
            logger.debug("Wait for the system running...");
        }

        if (MuleManager.isInstanciated()) {
            logger.debug("A Mule instance is already present");
            throw new Exception("A Mule instance is already present");
        }

        // read configuration file: eventswitch-config.xml
        final InputStreamReader isr;
        try {
            InputStream is = new FileInputStream(new File(JProp.getConfDir(), "eventswitch-config.xml"));
            isr = new InputStreamReader(is);
        } catch (FileNotFoundException e) {
            throw new Exception("Cannot read the config file 'eventswitch-config.xml'", e);
        }

        // Start Mule inside a Thread
        try {
            muleStarterThread = new Thread() {
                @Override
                public void run() {
                    try {
                        ConfigurationBuilder builder = new MuleXmlConfigurationBuilder();
                        ReaderResource configReader = new ReaderResource("eventswitch", isr);
                        // Can pass some properties to MuleManager (null = none)
                        UMOManager umo = builder.configure(new ReaderResource[]{configReader}, null);
                        // debug
                        if (logger.isDebugEnabled()) {
                            Map endpoints = umo.getEndpoints();
                            for (String key : (Set<String>) endpoints.keySet()) {
                                logger.debug(key);
                            }
                        }
                    } catch (Exception e) {
                        logger.error("Failed starting mule", e);
                        MuleManager.getInstance().dispose();
                    }
                }
            };

            muleStarterThread.setContextClassLoader(getClass().getClassLoader());
            muleStarterThread.start();
        } catch (Exception e) {
            throw new Exception("Failed configuring Mule", e);
        }
        logger.debug("MuleWrapper Started");
    }

    /**
     * Disposes of Mule.
     */
    @Invalidate
    public void stop() {
        logger.debug("");
        // Stop thread if still running
        if (muleStarterThread != null) {
            muleStarterThread.interrupt();
        }
        MuleManager.getInstance().dispose();
    }

}
